Package: ncvreg
Title: Regularization Paths for SCAD and MCP Penalized Regression
        Models
Version: 3.16.0
Authors@R: c(
  person("Patrick", "Breheny", role=c("aut","cre"),
    email="patrick-breheny@uiowa.edu", comment=c(ORCID="0000-0002-0650-1119")),
  person("Ryan", "Miller", role='aut', comment=c(ORCID="0000-0003-0446-9992")),
  person("Logan", "Harris", role='aut', comment=c(ORCID="0000-0001-8562-9534")))
Description: Fits regularization paths for linear regression, GLM, and Cox
  regression models using lasso or nonconvex penalties, in particular the
  minimax concave penalty (MCP) and smoothly clipped absolute deviation (SCAD)
  penalty, with options for additional L2 penalties (the "elastic net" idea).
  Utilities for carrying out cross-validation as well as post-fitting
  visualization, summarization, inference, and prediction are also provided.
  For more information, see Breheny and Huang (2011) <doi:10.1214/10-AOAS388>
  or visit the ncvreg homepage <https://pbreheny.github.io/ncvreg/>.
License: GPL-3
URL: https://pbreheny.github.io/ncvreg/,
        https://github.com/pbreheny/ncvreg
BugReports: https://github.com/pbreheny/ncvreg/issues
Depends: R (>= 4.1)
Suggests: ashr, knitr, parallel, rmarkdown, survival, tinytest
LazyData: true
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-11-04 21:08:21 UTC; pbreheny
Author: Patrick Breheny [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0650-1119>),
  Ryan Miller [aut] (ORCID: <https://orcid.org/0000-0003-0446-9992>),
  Logan Harris [aut] (ORCID: <https://orcid.org/0000-0001-8562-9534>)
Maintainer: Patrick Breheny <patrick-breheny@uiowa.edu>
Repository: CRAN
Date/Publication: 2025-11-06 06:10:33 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:38:38 UTC; unix
Archs: ncvreg.so.dSYM
