# Generated by roxygen2: do not edit by hand

S3method(AUC,cv.ncvsurv)
S3method(coef,cv.ncvreg)
S3method(coef,ncvreg)
S3method(logLik,ncvreg)
S3method(logLik,ncvsurv)
S3method(permres,ncvreg)
S3method(plot,cv.ncvreg)
S3method(plot,mfdr)
S3method(plot,ncvreg)
S3method(plot,ncvsurv.func)
S3method(predict,cv.ncvreg)
S3method(predict,cv.ncvsurv)
S3method(predict,ncvreg)
S3method(predict,ncvsurv)
S3method(print,summary.cv.ncvreg)
S3method(print,summary.ncvreg)
S3method(residuals,ncvreg)
S3method(summary,cv.ncvreg)
S3method(summary,ncvreg)
export(AUC)
export(assign_fold)
export(cv.ncvreg)
export(cv.ncvsurv)
export(intervals)
export(local_mfdr)
export(mfdr)
export(ncvfit)
export(ncvreg)
export(ncvsurv)
export(perm.ncvreg)
export(permres)
export(std)
import(graphics)
import(stats)
useDynLib(ncvreg, .registration = TRUE)
